@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 '''
 Provides IPython console widget.
 
@@ -12,13 +12,21 @@ available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import gtk, gobject
+import gi
+
+from gi.repository import Gtk as gtk
+from gi.repository import Gdk as gdk
+from gi.repository import GLib
+from gi.repository import Pango
+
+from pkg_resources import parse_version
+
 import re
 import sys
 import os
-import pango
-from StringIO import StringIO
-import thread
+
+from io import StringIO
+from functools import reduce
 
 try:
   import IPython
@@ -62,69 +70,189 @@ class IterableIPShell:
     @param input_func: Replacement for builtin raw_input()
     @type input_func: function
     '''
+    io = IPython.utils.io
     if input_func:
-      IPython.iplib.raw_input_original = input_func
-    if cin:
-      IPython.Shell.Term.cin = cin
-    if cout:
-      IPython.Shell.Term.cout = cout
-    if cerr:
-      IPython.Shell.Term.cerr = cerr
+      if parse_version(IPython.release.version) >= parse_version("1.2.1"):
+        IPython.terminal.interactiveshell.raw_input_original = input_func
+      else:
+        IPython.frontend.terminal.interactiveshell.raw_input_original = input_func
+    if IPython.version_info < (8,):
+      if cin:
+        io.stdin = io.IOStream(cin)
+      if cout:
+        io.stdout = io.IOStream(cout)
+      if cerr:
+        io.stderr = io.IOStream(cerr)
+    else:
+      if cin:
+        sys.stdin = cin
+      if cout:
+        sys.stdout = cout
+      if cerr:
+        sys.stderr = cerr
 
     # This is to get rid of the blockage that accurs during 
     # IPython.Shell.InteractiveShell.user_setup()
-    IPython.iplib.raw_input = lambda x: None
 
-    self.term = IPython.genutils.IOTerm(cin=cin, cout=cout, cerr=cerr)
+    io.raw_input = lambda x: None
+
     os.environ['TERM'] = 'dumb'
     excepthook = sys.excepthook 
-    self.IP = IPython.Shell.make_IPython(
-      argv,user_ns=user_ns,
-      user_global_ns=user_global_ns,
-      embedded=True,
-      shell_class=IPython.Shell.InteractiveShell)
+
+    if parse_version(IPython.release.version) >= parse_version('4.0.0'):
+        from traitlets.config.loader import Config
+    else:
+        from IPython.config.loader import Config
+    cfg = Config()
+    cfg.InteractiveShell.colors = "Linux"
+    cfg.Completer.use_jedi = False
+
+    if IPython.version_info < (8,):
+      # InteractiveShell's __init__ overwrites io.stdout,io.stderr with
+      # sys.stdout, sys.stderr, this makes sure they are right
+      old_stdout, old_stderr = sys.stdout, sys.stderr
+      sys.stdout, sys.stderr = io.stdout.stream, io.stderr.stream
+
+    # InteractiveShell inherits from SingletonConfigurable, so use instance()
+    #
+    if parse_version(IPython.release.version) >= parse_version("1.2.1"):
+      self.IP = IPython.terminal.embed.InteractiveShellEmbed.instance(\
+              config=cfg, user_ns=user_ns)
+    else:
+      self.IP = IPython.frontend.terminal.embed.InteractiveShellEmbed.instance(\
+              config=cfg, user_ns=user_ns)
+
+    if IPython.version_info < (8,):
+      sys.stdout, sys.stderr = old_stdout, old_stderr
+
     self.IP.system = lambda cmd: self.shell(self.IP.var_expand(cmd),
-                                            header='IPython system call: ',
-                                            verbose=self.IP.rc.system_verbose)
+                                            header='IPython system call: ')
+#                                            local_ns=user_ns)
+                                            #global_ns=user_global_ns)
+                                            #verbose=self.IP.rc.system_verbose)
+
+    self.IP.raw_input = input_func
     sys.excepthook = excepthook
     self.iter_more = 0
     self.history_level = 0
     self.complete_sep =  re.compile('[\s\{\}\[\]\(\)]')
+    self.updateNamespace({'exit':lambda:None})
+    self.updateNamespace({'quit':lambda:None})
+    if parse_version(IPython.release.version) < parse_version("5.0.0"):
+      self.IP.readline_startup_hook(self.IP.pre_readline)
+    # Workaround for updating namespace with sys.modules
+    #
+    self.__update_namespace()
+
+    # Avoid using input splitter when not really needed.
+    # Perhaps it could work even before 5.8.0
+    # But it definitely does not work any more with >= 7.0.0
+    self.no_input_splitter = parse_version(IPython.release.version) >= parse_version('5.8.0')
+    self.lines = []
+    self.indent_spaces = ''
+
+  def __update_namespace(self):
+    '''
+    Update self.IP namespace for autocompletion with sys.modules
+    '''
+    for k, v in list(sys.modules.items()):
+        if not '.' in k:
+          self.IP.user_ns.update({k:v})
 
   def execute(self):
     '''
     Executes the current line provided by the shell object.
     '''
     self.history_level = 0
-    orig_stdout = sys.stdout
-    sys.stdout = IPython.Shell.Term.cout
+
+    if IPython.version_info < (8,):
+      # this is needed because some functions in IPython use 'print' to print
+      # output (like 'who')
+
+      orig_stdout = sys.stdout
+      sys.stdout = IPython.utils.io.stdout
+
+      orig_stdin = sys.stdin
+      sys.stdin = IPython.utils.io.stdin;
+
+    self.prompt = self.generatePrompt(self.iter_more)
+
+    self.IP.hooks.pre_prompt_hook()
+    if self.iter_more:
+        try:
+            self.prompt = self.generatePrompt(True)
+        except:
+            self.IP.showtraceback()
+        if self.IP.autoindent:
+            self.IP.rl_do_indent = True
+
     try:
-      line = self.IP.raw_input(None, self.iter_more)
-      if self.IP.autoindent:
-        self.IP.readline_startup_hook(None)
+      line = self.IP.raw_input(self.prompt)
     except KeyboardInterrupt:
       self.IP.write('\nKeyboardInterrupt\n')
-      self.IP.resetbuffer()
-      # keep cache in sync with the prompt counter:
-      self.IP.outputcache.prompt_count -= 1
-        
-      if self.IP.autoindent:
-        self.IP.indent_current_nsp = 0
-      self.iter_more = 0
+      if self.no_input_splitter:
+        self.lines = []
+      else:
+        self.IP.input_splitter.reset()
     except:
       self.IP.showtraceback()
     else:
-      self.iter_more = self.IP.push(line)
-      if (self.IP.SyntaxTB.last_syntax_error and
-          self.IP.rc.autoedit_syntax):
-        self.IP.edit_syntax_error()
-    if self.iter_more:
-      self.prompt = str(self.IP.outputcache.prompt2).strip()
-      if self.IP.autoindent:
-        self.IP.readline_startup_hook(self.IP.pre_readline)
+      if self.no_input_splitter:
+        self.lines.append(line)
+        (status, self.indent_spaces) = self.IP.check_complete('\n'.join(self.lines))
+        self.iter_more = status == 'incomplete'
+      else:
+        self.IP.input_splitter.push(line)
+        self.iter_more = self.IP.input_splitter.push_accepts_more()
+      if not self.iter_more:
+          if self.no_input_splitter:
+            source_raw = '\n'.join(self.lines)
+            self.lines = []
+          elif parse_version(IPython.release.version) >= parse_version("2.0.0-dev"):
+            source_raw = self.IP.input_splitter.raw_reset()
+          else:
+            source_raw = self.IP.input_splitter.source_raw_reset()[1]
+          self.IP.run_cell(source_raw, store_history=True)
+          self.IP.rl_do_indent = False
+      else:
+          # TODO: Auto-indent
+          #
+          self.IP.rl_do_indent = True
+          pass
+      self.prompt = self.generatePrompt(self.iter_more)
+
+    if IPython.version_info < (8,):
+      sys.stdout = orig_stdout
+      sys.stdin = orig_stdin
+
+  def generatePrompt(self, is_continuation):
+    '''
+    Generate prompt depending on is_continuation value
+
+    @param is_continuation
+    @type is_continuation: boolean 
+
+    @return: The prompt string representation
+    @rtype: string
+
+    '''
+
+    # Backwards compatibility with ipyton-0.11
+    #
+    ver = IPython.__version__
+    if ver[0:4] == '0.11':
+        prompt = self.IP.hooks.generate_prompt(is_continuation)
+    elif parse_version(IPython.release.version) < parse_version("5.0.0"):
+        if is_continuation:
+            prompt = self.IP.prompt_manager.render('in2')
+        else:
+            prompt = self.IP.prompt_manager.render('in')
     else:
-      self.prompt = str(self.IP.outputcache.prompt1).strip()
-    sys.stdout = orig_stdout
+        # TODO: update to IPython 5.x and later
+        prompt = "In [%d]: " % self.IP.execution_count
+
+    return prompt
+
 
   def historyBack(self):
     '''
@@ -134,6 +262,8 @@ class IterableIPShell:
     @rtype: string
     '''
     self.history_level -= 1
+    if not self._getHistory():
+      self.history_level +=1
     return self._getHistory()
   
   def historyForward(self):
@@ -143,7 +273,8 @@ class IterableIPShell:
     @return: The command string.
     @rtype: string
     '''
-    self.history_level += 1
+    if self.history_level < 0:
+      self.history_level += 1
     return self._getHistory()
   
   def _getHistory(self):
@@ -156,7 +287,6 @@ class IterableIPShell:
     try:
       rv = self.IP.user_ns['In'][self.history_level].strip('\n')
     except IndexError:
-      self.history_level = 0
       rv = ''
     return rv
 
@@ -181,7 +311,11 @@ class IterableIPShell:
     @rtype: tuple
     '''
     split_line = self.complete_sep.split(line)
-    possibilities = self.IP.complete(split_line[-1])
+    if split_line[-1]:
+      possibilities = self.IP.complete(split_line[-1])
+    else:
+      completed = line
+      possibilities = ['', []]
     if possibilities:
       def _commonPrefix(str1, str2):
         '''
@@ -199,11 +333,14 @@ class IterableIPShell:
           if not str2.startswith(str1[:i+1]):
             return str1[:i]
         return str1
-      common_prefix = reduce(_commonPrefix, possibilities)
-      completed = line[:-len(split_line[-1])]+common_prefix
+      if possibilities[1]:
+        common_prefix = reduce(_commonPrefix, possibilities[1]) or split_line[-1]
+        completed = line[:-len(split_line[-1])]+common_prefix
+      else:
+        completed = line
     else:
       completed = line
-    return completed, possibilities
+    return completed, possibilities[1]
   
 
   def shell(self, cmd,verbose=0,debug=0,header=''):
@@ -220,11 +357,11 @@ class IterableIPShell:
     @type header: string
     '''
     stat = 0
-    if verbose or debug: print header+cmd
+    if verbose or debug: print(header+cmd)
     # flush stdout so we don't mangle python's buffering
     if not debug:
       input, output = os.popen4(cmd)
-      print output.read()
+      print(output.read())
       output.close()
       input.close()
 
@@ -232,7 +369,8 @@ class ConsoleView(gtk.TextView):
   '''
   Specialized text view for console-like workflow.
 
-  @cvar ANSI_COLORS: Mapping of terminal colors to X11 names.
+  @cvar ANSI_COLORS: Mapping of terminal control sequence values to
+                     tuples containing foreground and background color names.
   @type ANSI_COLORS: dictionary
 
   @ivar text_buffer: Widget's text buffer.
@@ -244,21 +382,51 @@ class ConsoleView(gtk.TextView):
   @ivar line_start: Start of command line mark.
   @type line_start: gtk.TextMark
   '''
-  ANSI_COLORS =  {'0;30': 'Black',     '0;31': 'Red',
-                  '0;32': 'Green',     '0;33': 'Brown',
-                  '0;34': 'Blue',      '0;35': 'Purple',
-                  '0;36': 'Cyan',      '0;37': 'LightGray',
-                  '1;30': 'DarkGray',  '1;31': 'DarkRed',
-                  '1;32': 'SeaGreen',  '1;33': 'Yellow',
-                  '1;34': 'LightBlue', '1;35': 'MediumPurple',
-                  '1;36': 'LightCyan', '1;37': 'White'}
+  ANSI_COLORS = {'0;30': ('Black', None),
+                 '0;31': ('Red', None),
+                 '0;32': ('Green', None),
+                 '0;33': ('Brown', None),
+                 '0;34': ('Blue', None),
+                 '0;35': ('Purple', None),
+                 '0;36': ('Cyan', None),
+                 '0;37': ('LightGray', None),
+                 '1;30': ('DarkGray', None),
+                 '1;31': ('DarkRed', None),
+                 '1;32': ('SeaGreen', None),
+                 '1;33': ('Yellow', None),
+                 '1;34': ('LightBlue', None),
+                 '1;35': ('MediumPurple', None),
+                 '1;36': ('LightCyan', None),
+                 '1;37': ('White', None),
+                 '38;5;124;43': ('DarkRed', 'Yellow'),
+                 '38;5;241': ('Gray', None),
+                 '38;5;241;43': ('Gray', 'Yellow'),
+                 '38;5;28;43;01': ('Green', 'Yellow'),
+                 '39': ('Black', None),
+                 '39;49': ('Red', 'White'),
+                 '39;49;00': ('Red', 'White'),
+                 '43': (None, 'Yellow'),
+                 '49': (None, 'White')}
 
   def __init__(self):
     '''
     Initialize console view.
     '''
     gtk.TextView.__init__(self)
-    self.modify_font(pango.FontDescription('Mono'))
+    pango_ctx = self.get_pango_context()
+    chosen = None
+    for f in pango_ctx.list_families():
+        name = f.get_name()
+        # These are known to show e.g U+FFFC
+        if name in [ "Courier New", "Courier Mono" ]:
+            chosen = name
+            break
+        if name in [ "Liberation Sans" ]:
+            chosen = name
+            # But prefer a monospace one if possible
+    if chosen == None:
+        chosen = "Mono"
+    self.modify_font(Pango.FontDescription(chosen))
     self.set_cursor_visible(True)
     self.text_buffer = self.get_buffer()
     self.mark = self.text_buffer.create_mark('scroll_mark', 
@@ -266,7 +434,8 @@ class ConsoleView(gtk.TextView):
                                              False)
     for code in self.ANSI_COLORS:
       self.text_buffer.create_tag(code, 
-                                  foreground=self.ANSI_COLORS[code], 
+                                  foreground=self.ANSI_COLORS[code][0],
+                                  background=self.ANSI_COLORS[code][1],
                                   weight=700)
     self.text_buffer.create_tag('0')
     self.text_buffer.create_tag('notouch', editable=False)
@@ -277,7 +446,7 @@ class ConsoleView(gtk.TextView):
     self.connect('key-press-event', self.onKeyPress)
     
   def write(self, text, editable=False):
-    gobject.idle_add(self._write, text, editable)
+    GLib.idle_add(self._write, text, editable)
 
   def _write(self, text, editable=False):
     '''
@@ -311,7 +480,7 @@ class ConsoleView(gtk.TextView):
 
 
   def showPrompt(self, prompt):
-    gobject.idle_add(self._showPrompt, prompt)
+    GLib.idle_add(self._showPrompt, prompt)
 
   def _showPrompt(self, prompt):
     '''
@@ -325,7 +494,7 @@ class ConsoleView(gtk.TextView):
                                self.text_buffer.get_end_iter())
 
   def changeLine(self, text):
-    gobject.idle_add(self._changeLine, text)
+    GLib.idle_add(self._changeLine, text)
 
   def _changeLine(self, text):
     '''
@@ -352,7 +521,7 @@ class ConsoleView(gtk.TextView):
     return rv
 
   def showReturned(self, text):
-    gobject.idle_add(self._showReturned, text)
+    GLib.idle_add(self._showReturned, text)
 
   def _showReturned(self, text):
     '''
@@ -371,9 +540,16 @@ class ConsoleView(gtk.TextView):
     if text:
       self._write('\n')
     self._showPrompt(self.prompt)
-    self.text_buffer.move_mark(self.line_start,self.text_buffer.get_end_iter())
+    self.text_buffer.move_mark(self.line_start, self.text_buffer.get_end_iter())
     self.text_buffer.place_cursor(self.text_buffer.get_end_iter())
 
+    if self.IP.rl_do_indent:
+      if self.no_input_splitter:
+        indentation = self.indent_spaces
+      else:
+        indentation = self.IP.input_splitter.indent_spaces * ' '
+      self.text_buffer.insert_at_cursor(indentation)
+
   def onKeyPress(self, widget, event):
     '''
     Key press callback used for correcting behavior for console-like 
@@ -393,19 +569,38 @@ class ConsoleView(gtk.TextView):
     selection_mark = self.text_buffer.get_selection_bound()
     selection_iter = self.text_buffer.get_iter_at_mark(selection_mark)
     start_iter = self.text_buffer.get_iter_at_mark(self.line_start)
-    if event.keyval == gtk.keysyms.Home:
-      if event.state & gtk.gdk.CONTROL_MASK or event.state & gtk.gdk.MOD1_MASK:
+    if event.keyval == gdk.KEY_Home:
+      if event.state & gdk.ModifierType.CONTROL_MASK or \
+              event.state & gdk.ModifierType.MOD1_MASK:
         pass
-      elif event.state & gtk.gdk.SHIFT_MASK:
+      elif event.state & gdk.ModifierType.SHIFT_MASK:
         self.text_buffer.move_mark(insert_mark, start_iter)
         return True
       else:
         self.text_buffer.place_cursor(start_iter)
         return True
-    elif event.keyval == gtk.keysyms.Left:
+    elif event.keyval == gdk.KEY_Left:
       insert_iter.backward_cursor_position()
       if not insert_iter.editable(True):
         return True
+    elif event.state & gdk.ModifierType.CONTROL_MASK and event.keyval in [ord('L'), ord('l')]:
+        # clear previous output on Ctrl+L, but remember current input line + cursor position
+        cursor_offset = self.text_buffer.get_property('cursor-position')
+        cursor_pos_in_line = cursor_offset - start_iter.get_offset() + len(self.prompt)
+        current_input = self.text_buffer.get_text(start_iter, self.text_buffer.get_end_iter(), False)
+        self.text_buffer.set_text(self.prompt + current_input)
+        self.text_buffer.move_mark(self.line_start, self.text_buffer.get_iter_at_offset(len(self.prompt)))
+        self.text_buffer.place_cursor(self.text_buffer.get_iter_at_offset(cursor_pos_in_line))
+        return True
+    elif event.state & gdk.ModifierType.CONTROL_MASK and event.keyval in [gdk.KEY_k, gdk.KEY_K]:
+      # clear text after input cursor on Ctrl+K
+      if insert_iter.editable(True):
+        self.text_buffer.delete(insert_iter, self.text_buffer.get_end_iter())
+      return True
+    elif event.state & gdk.ModifierType.CONTROL_MASK and event.keyval == gdk.KEY_C:
+      # copy selection on Ctrl+C (upper-case 'C' only)
+      self.text_buffer.copy_clipboard(gtk.Clipboard.get(gdk.SELECTION_CLIPBOARD))
+      return True
     elif not event.string:
       pass
     elif start_iter.compare(insert_iter) <= 0 and \
@@ -438,13 +633,14 @@ class IPythonView(ConsoleView, IterableIPShell):
     '''
     ConsoleView.__init__(self)
     self.cout = StringIO()
-    IterableIPShell.__init__(self, cout=self.cout,cerr=self.cout, 
+    IterableIPShell.__init__(self, cout=self.cout, cerr=self.cout,
                              input_func=self.raw_input)
 #    self.connect('key_press_event', self.keyPress)
+    self.interrupt = False
     self.execute()
+    self.prompt = self.generatePrompt(False)
     self.cout.truncate(0)
     self.showPrompt(self.prompt)
-    self.interrupt = False
 
   def raw_input(self, prompt=''):
     '''
@@ -474,20 +670,20 @@ class IPythonView(ConsoleView, IterableIPShell):
     @return: True if event should not trickle.
     @rtype: boolean
     '''
-    if event.state & gtk.gdk.CONTROL_MASK and event.keyval == 99:
+    if event.state & gdk.ModifierType.CONTROL_MASK and event.keyval == gdk.KEY_c:
       self.interrupt = True
       self._processLine()
       return True
-    elif event.keyval == gtk.keysyms.Return:
+    elif event.keyval == gdk.KEY_Return:
       self._processLine()
       return True
-    elif event.keyval == gtk.keysyms.Up:
+    elif event.keyval == gdk.KEY_Up:
       self.changeLine(self.historyBack())
       return True
-    elif event.keyval == gtk.keysyms.Down:
+    elif event.keyval == gdk.KEY_Down:
       self.changeLine(self.historyForward())
       return True
-    elif event.keyval == gtk.keysyms.Tab:
+    elif event.keyval == gdk.KEY_Tab:
       if not self.getCurrentLine().strip():
         return False
       completed, possibilities = self.complete(self.getCurrentLine())
@@ -510,4 +706,13 @@ class IPythonView(ConsoleView, IterableIPShell):
     if rv: rv = rv.strip('\n')
     self.showReturned(rv)
     self.cout.truncate(0)
-    
+    self.cout.seek(0)
+ 
+if __name__ == "__main__":
+  window = gtk.Window()
+  window.set_default_size(640, 320)
+  window.connect('delete-event', lambda x, y: gtk.main_quit())
+  window.add(IPythonView())
+  window.show_all()
+  gtk.main()
+ 
