@@ -12,14 +12,20 @@ available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import gtk
-import gobject
+from gi.repository import Gtk as gtk
+from gi.repository import Gdk as gdk
+from gi.repository import GdkPixbuf
+from gi.repository import GLib
+from gi.repository import GObject
+
 import pyatspi
-import atk, os
-from icons import getIcon
-from node import Node
-from tools import Tools
-from i18n import _
+import os
+from . import ui_manager
+from time import sleep
+from .icons import getIcon
+from .node import Node
+from .tools import ToolsAccessor, getTreePathBoundingBox
+from .i18n import _
 
 COL_ICON = 0
 COL_NAME = 1
@@ -29,7 +35,7 @@ COL_FILLED = 4
 COL_DUMMY = 5
 COL_ACC = 6
 
-class AccessibleModel(gtk.TreeStore, Tools):
+class AccessibleModel(gtk.TreeStore, ToolsAccessor):
   '''
   Stores the desktop accessible tree. Only populates sections of the tree
   that are being viewed. This cuts short on a lot of potential overhead.
@@ -42,16 +48,16 @@ class AccessibleModel(gtk.TreeStore, Tools):
   @type acc_cache: list
   '''
   __gsignals__ = {'row-filled' : 
-                  (gobject.SIGNAL_RUN_FIRST,
-                   gobject.TYPE_NONE, 
-                   (gobject.TYPE_PYOBJECT,)),
+                  (GObject.SignalFlags.RUN_FIRST,
+                   None, 
+                   (GObject.TYPE_PYOBJECT,)),
                   'start-populating' :
-                    (gobject.SIGNAL_RUN_FIRST,
-                     gobject.TYPE_NONE, 
+                    (GObject.SignalFlags.RUN_FIRST,
+                     None,
                      ()),
                   'end-populating' :
-                    (gobject.SIGNAL_RUN_FIRST,
-                     gobject.TYPE_NONE, 
+                    (GObject.SignalFlags.RUN_FIRST,
+                     None, 
                      ())}
 
   def __init__(self, desktop_acc):
@@ -60,7 +66,7 @@ class AccessibleModel(gtk.TreeStore, Tools):
     Connects required signals.
     '''
     self.acc_cache = [desktop_acc]
-    gtk.TreeStore.__init__(self, gtk.gdk.Pixbuf, str, str, str, bool, bool, object)
+    gtk.TreeStore.__init__(self, GdkPixbuf.Pixbuf, str, str, int, bool, bool, object)
     self.connect('row-changed', self._onRowChanged)
     self.connect('row-filled', self._onRowFilled)
     self.desktop = desktop_acc
@@ -173,7 +179,7 @@ class AccessibleModel(gtk.TreeStore, Tools):
     @param iter: Th iter of the row that needs to be pre-populated
     @type iter: L{gtk.TreeIter}
     '''
-    if (parent and parent.childCount > 0 and 
+    if (parent and self.children_number(parent) > 0 and
         not self.isMyApp(parent) and self.iter_n_children(iter) == 0):
       row = self._buildRow(None, True)
       self.append(iter, row)
@@ -186,13 +192,55 @@ class AccessibleModel(gtk.TreeStore, Tools):
     @type iter: L{gtk.TreeIter}
     '''
     if iter:
-      row_reference = gtk.TreeRowReference(self, self.get_path(iter))
+      tree_path = self.get_path(iter)
+      path = tuple(tree_path.get_indices())
+
+      row_reference = gtk.TreeRowReference.new(self, tree_path)
+      #row_reference = gtk.TreeRowReference()
     else:
       row_reference = None
     self._populating_tasks += 1
     if self._populating_tasks == 1:
       self.emit('start-populating')
-    gobject.idle_add(self._popOnIdle, row_reference)
+    GLib.idle_add(self._popOnIdle, row_reference)
+
+  def _childrenIndexesInParent(self, accessible):
+    '''
+    Gets ids from children (either all of them or only those that are not leaves).
+
+    @param accessible: An accessible from which we want to get children ids.
+    @type accessible: L{Accessibility.Accessible}
+
+    @return: A list with children ids.
+    @rtype: list
+    '''
+    return [i for i in range(accessible.childCount)]
+
+  def _selectChild(self, parent, index):
+    '''
+    Returns a child id.
+
+    @param parent: An accessible from which we want to select a child id.
+    @type parent: L{Accessibility.Accessible}
+    @param index: An index to retrieve the child id. In some cases, it is equal to the id.
+    @type index: integer
+
+    @return: A child id.
+    @rtype: integer
+    '''
+    return index
+
+  def children_number(self, parent):
+    '''
+    Returns how many children an accessible has (either all of them or only those that are not leaves).
+
+    @param parent: An accessible from which we want to get the number of children.
+    @type parent: L{Accessibility.Accessible}
+
+    @return: The number of children (including leaves or not) an accessible has.
+    @rtype: integer
+    '''
+    return parent.childCount
 
   def _popOnIdle(self, row_reference):
     '''
@@ -214,12 +262,14 @@ class AccessibleModel(gtk.TreeStore, Tools):
         return False
       iter = self.get_iter(row_reference.get_path())
       parent = self[iter][COL_ACC]
-      if self[(row_reference.get_path())+(0,)][COL_DUMMY]:
+      tree_path = row_reference.get_path()
+      path = tuple(tree_path.get_indices())
+      if self[path+(0,)][COL_DUMMY]:
         remove_iter = self.iter_children(iter)
 
     already_populated_num = self.iter_n_children(iter)
 
-    if already_populated_num >= parent.childCount and \
+    if already_populated_num >= self.children_number(parent) and \
           not remove_iter:
       if iter:
         self[iter][COL_FILLED] = True
@@ -229,7 +279,8 @@ class AccessibleModel(gtk.TreeStore, Tools):
     elif remove_iter:
       already_populated_num -= 1
     try:
-      child = parent.getChildAtIndex(already_populated_num)
+      index = self._selectChild(parent, already_populated_num)
+      child = parent.getChildAtIndex(index)
     except LookupError:
       child = None
 
@@ -271,7 +322,7 @@ class AccessibleModel(gtk.TreeStore, Tools):
     @return: True if entire path is populated.
     @rtype: boolean
     '''
-    for i in xrange(1, len(path)):
+    for i in range(1, len(path)):
       if not self[path[:i]][COL_FILLED]:
         self.popLevel(self.get_iter(path[:i]))
         return False
@@ -288,11 +339,24 @@ class AccessibleModel(gtk.TreeStore, Tools):
     @type iter: gtk.TreeIter
     '''
     if iter and self._path_to_populate:
-      path = self.get_path(iter)
+      tree_path = self.get_path(iter)
+      path = tuple(tree_path.get_indices())
       if self._path_to_populate[:len(path)] == path:
         if self._walkThroughFilled(self._path_to_populate):
           self._path_to_populate = None
 
+  def getIndexInParent(self, child):
+    '''
+    Returns a position of a child in its parent.
+
+    @param child: Accessible for which we want to determine the index.
+    @type child: L{Accessibility.Accessible}
+
+    @return: The child's index or -1 if it wasn't found
+    @rtype: integer
+    '''
+    return child.getIndexInParent()
+
   def getAccPath(self, acc):
     '''
     Get the tree path that a given accessible should have.
@@ -307,19 +371,21 @@ class AccessibleModel(gtk.TreeStore, Tools):
     '''
     path = ()
     child = acc
-    while child.parent:
+    while child.get_parent():
       try:
-        index_in_parent = child.getIndexInParent()
+        index_in_parent = self.getIndexInParent(child)
         if index_in_parent < 0:
           break
         path = (index_in_parent,) + path
-      except Exception, e:
+      except Exception as e:
         return None
-      child = child.parent
+      child = child.get_parent()
+
     try:
       path = (list(self.desktop).index(child),) + path
-    except Exception, e:
+    except Exception as e:
       return None
+
     return path
 
   def _buildRow(self, accessible, dummy=False):
@@ -335,9 +401,14 @@ class AccessibleModel(gtk.TreeStore, Tools):
     '''
     if accessible is not None:
       icon = getIcon(accessible)
-      name = accessible.name
-      role = accessible.getLocalizedRoleName()
-      count = str(accessible.childCount)
+      if self.isMyApp(accessible):
+        name = "Accerciser"
+        role = "accerciser"
+        count = 0
+      else:
+        name = accessible.name
+        role = accessible.getLocalizedRoleName()
+        count = accessible.childCount
     else:
       icon = None
       if not dummy:
@@ -345,10 +416,10 @@ class AccessibleModel(gtk.TreeStore, Tools):
       else:
         name = None
       role = None
-      count = None
+      count = 0
     return [icon, name, role, count, False, dummy, accessible]
 
-class AccessibleTreeView(gtk.TreeView, Tools):
+class AccessibleTreeView(gtk.TreeView, ToolsAccessor):
   '''
   The treeview for the desktop's accessibles. The treeview's model (L{AccessibleModel}
   is only populated when the treeview is traversed and nodes are expanded. This class
@@ -382,31 +453,32 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     self.connect('row-activated', self._onRowActivated)
 
     self.model = AccessibleModel(self.desktop)
-    self.set_model(self.model)
+    self.filter = self.model.filter_new()
+    self.filter.set_visible_func(self._filterNoChildApps, data=None)
+    self.set_model(self.filter)
+
     crt = gtk.CellRendererText()
     crp = gtk.CellRendererPixbuf()
     tvc = gtk.TreeViewColumn(_('Name'))
     tvc.pack_start(crp, False)
     tvc.pack_start(crt, True)
-    tvc.set_attributes(crp, pixbuf=COL_ICON)
-    tvc.set_attributes(crt, text=COL_NAME)
+    tvc.add_attribute(crp, 'pixbuf', COL_ICON)
+    tvc.add_attribute(crt, 'text', COL_NAME)
     tvc.set_resizable(True)
-    tvc.set_cell_data_func(crt, self._accCellDataFunc)
-    tvc.set_cell_data_func(crp, self._accCellDataFunc)
     self.append_column(tvc)
+
     crt= gtk.CellRendererText()
     tvc = gtk.TreeViewColumn(_('Role'))
     tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=COL_ROLE)
+    tvc.add_attribute(crt, 'text', COL_ROLE)
     tvc.set_resizable(True)
-    tvc.set_cell_data_func(crt, self._accCellDataFunc)
     self.append_column(tvc)
+
     crt = gtk.CellRendererText()
     tvc = gtk.TreeViewColumn(_('Children'))
     tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=COL_CHILDCOUNT)
+    tvc.add_attribute(crt, 'text', COL_CHILDCOUNT)
     tvc.set_resizable(True)
-    tvc.set_cell_data_func(crt, self._accCellDataFunc)
     self.append_column(tvc)
 
     self.model.connect('row-filled', self._onRowFilled)
@@ -419,24 +491,112 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     selection = self.get_selection()
     selection.unselect_all()
     selection.connect('changed', self._onSelectionChanged)
-    selection.set_select_function(self._selectFunc)
     self.connect('row-expanded', self._onExpanded)
+
     pyatspi.Registry.registerEventListener(self._accEventChildChanged, 
                                            'object:children-changed')
 
-    self.action_group = gtk.ActionGroup('TreeActions')
+    pyatspi.Registry.registerEventListener(
+        self._accEventNameChanged, 
+        'object:property-change:accessible-name')
+
+    self._hide_leaves = True
+    self.action_group = gtk.ActionGroup.new('TreeActions')
+    self.action_group.add_toggle_actions(
+      [('HideShowLeaves', None, _('_Show Applications without children'), None,
+        None, self._onHideShowLeaves, False)])
     self.action_group.add_actions([
-        ('RefreshAll', gtk.STOCK_REFRESH, None,
-         None, 'Refresh all', self._refreshTopLevel),
+        ('RefreshAll', gtk.STOCK_REFRESH, _('_Refresh Registry'),
+        # Translators: Appears as tooltip
+        #
+         None, _('Refresh all'), self._refreshTopLevel),
         # Translators: Refresh current tree node's children.
         #
-        ('RefreshCurrent', gtk.STOCK_JUMP_TO, _('Refresh _Current'),
-         None, 'Refresh selected node\'s children', self._refreshCurrentLevel)])  
+        ('RefreshCurrent', gtk.STOCK_JUMP_TO, _('Refresh _Node'),
+        # Translators: Appears as tooltip
+        #
+         None, _('Refresh selected node’s children'), 
+         self._refreshCurrentLevel)])  
 
     self.refresh_current_action = self.action_group.get_action('RefreshCurrent')
     self.refresh_current_action.set_sensitive(False)
 
-  def _refreshTopLevel(self, action=None):
+    self.connect('popup-menu', self._onPopup)
+    self.connect('button-press-event', self._onPopup)
+    self.connect('key-press-event', self._onKeyPress)
+
+    self.connect('cursor-changed', self._onCursorChanged)
+
+  def _filterNoChildApps(self, model, iter, data):
+    '''
+    Filter all rows of applications without children
+    '''
+    return not self._hide_leaves or model.iter_parent(iter) != None \
+      or model[iter][COL_CHILDCOUNT] != 0 or self.isMyApp(model[iter][COL_ACC])
+
+  def _onCursorChanged(self, tree):
+    '''
+    Set sensitivity of refresh function only if the tree cursor is 
+    on an accessible.
+    '''
+    path = self.get_cursor()[0]
+    self.refresh_current_action.set_sensitive(path is not None)      
+
+  def _onKeyPress(self, w, event):
+    '''
+    Expand or collapse a row on Left/Right key-press
+    '''
+    if event.state & \
+        (gdk.ModifierType.SHIFT_MASK
+        |gdk.ModifierType.CONTROL_MASK
+        |gdk.ModifierType.MOD1_MASK) == 0:
+      path, col = self.get_cursor()
+      if path is not None:
+        if event.keyval == gdk.KEY_Left:
+          if not self.collapse_row(path):
+            # if we ccouldn't collapse the current row, go to the parent
+            if path.up():
+              self.set_cursor(path)
+          return True
+        elif event.keyval == gdk.KEY_Right:
+          self.expand_row(path, False)
+          return True
+    return False
+
+  def _onPopup(self, w, event=None):
+    '''
+    Callback for popup button or right mouse button. Brings up a context
+    menu.
+    '''
+    if event:
+      if event.button != 3:
+        return False
+      info = self.get_path_at_pos(int(event.x), int(event.y))
+      if info is None:
+        return False
+      path = info[0]
+      if self.isMyApp(self.filter[self.filter.get_iter(path)][COL_ACC]):
+        return False
+      selection = self.get_selection()
+      selection.set_mode(gtk.SelectionMode.NONE)
+      self.set_cursor(path, None, False)
+      selection.set_mode(gtk.SelectionMode.SINGLE)      
+      time = event.time
+      button = event.button
+      func = None
+      extra_data = None
+    else:
+      path, col= self.get_cursor()
+      time = gtk.get_current_event_time()
+      button = 0
+      extra_data = getTreePathBoundingBox(self, path, col)
+      func = lambda m, b: (b.x, b.y + (b.height/2), True)
+      
+    menu = ui_manager.uimanager.get_widget(ui_manager.POPUP_MENU_PATH)
+    menu.popup(None, None, func, extra_data, button, time)
+    return True
+
+  def _refreshTopLevel(self, action=None, data=None):
     '''
     Refreshes the entire tree at the desktop level.
 
@@ -447,20 +607,31 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     self.model.popLevel(None)
     # iter over all apps in the desktop too
     
-  def _refreshCurrentLevel(self, action):
+  def _refreshCurrentLevel(self, action, data=None):
     '''
     Refreshes the current level. Selects and expands the parent of the level.
 
     @param action: Action object that emitted this signal, if any.
     @type: gtk.Action
     '''
-    selection = self.get_selection()
-    model, iter = selection.get_selected()
-    is_expanded = self.row_expanded(self.model.get_path(iter))
-    self._refreshChildren(iter)
+    path = self.get_cursor()[0]
+    if path == None:
+      return
+    filter_iter = self.filter.get_iter(path)
+    if self.isMyApp(self.filter[filter_iter][COL_ACC]):
+      return
+    is_expanded = self.row_expanded(path)
+    self._refreshChildren(self.filter.convert_iter_to_child_iter(filter_iter))
     if is_expanded:
-      self.expand_row(self.model.get_path(iter), False)
-      self._onExpanded(self, iter, self.model.get_path(iter))
+      self.expand_row(path, False)
+      self._onExpanded(self, self.filter.get_iter(path), path)
+
+  def _onHideShowLeaves(self, option):
+    '''
+    Hides/Shows all leaves (accessibles with no children) from the accessible treeview.
+    '''
+    self._hide_leaves = not self._hide_leaves
+    self.filter.refilter()
 
   def _onExpanded(self, treeview, iter, path):
     '''
@@ -475,11 +646,30 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @type path: tuple
     '''
     # don't repopulate if it has been filled before
-    if self.model[iter][COL_FILLED]:
+    if self.filter[iter][COL_FILLED]:
       return
-    acc = self.model[iter][COL_ACC]
     # populate this level
-    self.model.popLevel(iter)
+    self.model.popLevel(self.filter.convert_iter_to_child_iter(iter))
+
+  def _accEventNameChanged(self, event):
+    '''
+    Event handler for "object:property-change:accessible-name". 
+    Updates the treeview accordingly.
+    
+    @param event: The event which triggered this handler.
+    @type event: L{pyatspi.event.Event}
+    '''
+    if self.isMyApp(event.source) or event.source == self.desktop:
+      # Bad karma
+      return
+    if self.model.isInModel(event.source):
+      try:
+        path = self.model.getAccPath(event.source)
+        iter = self.model.get_iter(path)
+      except:
+          pass
+      else:
+          self.model[iter][COL_NAME] = event.source.name
 
   def _accEventChildChanged(self, event):
     '''
@@ -497,6 +687,7 @@ class AccessibleTreeView(gtk.TreeView, Tools):
         iter = self.model.get_iter(path)
       except:
         iter = None
+        path = None
       if (event.source == self.desktop) or \
             (path and self.model[path][COL_FILLED]):
         if event.type.minor == 'add':
@@ -504,7 +695,7 @@ class AccessibleTreeView(gtk.TreeView, Tools):
         elif event.type.minor == 'remove':
           self._removeChild(iter, event.source)
       if iter and self.model.iter_is_valid(iter):
-        self.model[iter][COL_CHILDCOUNT] = str(event.source.childCount)
+        self.model[iter][COL_CHILDCOUNT] = event.source.childCount
 
   def _addChild(self, iter, parent):
     '''
@@ -517,6 +708,7 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     '''
     old_children = set(self.model.getChildrenAccs(iter))
     new_children = set(list(parent))
+
     added = new_children.difference(old_children)
     try:
       new_child = added.pop()
@@ -526,11 +718,11 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     if new_child is None:
       self.model.append(iter, row)
     else:
-      self.model.insert(iter, new_child.getIndexInParent(), row)
+      self.model.insert(iter, self.model.getIndexInParent(new_child), row)
       # We do this because an application won't have an icon loaded in 
       # the window manager when it is first registered to at-spi
       if new_child == new_child.getApplication():
-        gobject.timeout_add(1000, self._refreshIcon, new_child)
+        GLib.timeout_add(1000, self._refreshIcon, new_child)
     
   def _refreshIcon(self, app):
     '''
@@ -559,15 +751,25 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     if parent_iter:
       iter = self.model.iter_children(parent_iter)
     else:
-      iter = self.model.get_iter_root()
+      iter = self.model.get_iter_first()
+
     while iter:
       if self.model[iter][COL_ACC] not in parent:
+        cursor_path = self.get_cursor()[0]
+        if cursor_path != None:
+          (res, filter_iter) = self.filter.convert_child_iter_to_iter(iter)
+          if res:
+            filter_path = self.filter.get_path(filter_iter)
+            if filter_path.is_ancestor(cursor_path):
+              cursor_path = filter_path
+            if 0 == filter_path.compare(cursor_path):
+              if filter_path.prev() or filter_path.up():
+                self.set_cursor(filter_path, None, False)
         if not self.model.remove(iter):
           break
       else:
         iter = self.model.iter_next(iter)
 
-
   def _refreshChildren(self, iter):
     '''
     Remove all of a given node's children from the model.
@@ -592,7 +794,8 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     '''
     selection = self.get_selection()
     model, iter = selection.get_selected()
-    self._refreshChildren(iter)
+    if not self.isMyApp(model[iter][COL_ACC]):
+      self._refreshChildren(iter)
 
   def _onSelectionChanged(self, selection):
     '''
@@ -606,17 +809,17 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     model, iter = selection.get_selected()
     if iter:
       new_acc = model[iter][COL_ACC]
-      self.refresh_current_action.set_sensitive(True)
     else:
       new_acc = self.desktop
-      self.refresh_current_action.set_sensitive(False)
     if new_acc == self.node.acc:
       return
     self.node.handler_block(self._changed_handler)
     self.node.update(new_acc)
     self.node.handler_unblock(self._changed_handler)
     if iter:
-      path = model.get_path(iter)
+      tree_path = model.get_path(iter)
+      path = tuple(tree_path.get_indices())
+
       self.node.tree_path = list(path[1:])
 
   def _onAccChanged(self, node, acc):
@@ -653,7 +856,16 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     if dummy:
       self._path_to_expand = path
       self.model.popToPath(path)
-    else:
+      # process all pending events, so the path actually
+      # gets populated via the corresponding idles
+      main_loop_context = GLib.MainContext.default()
+      while main_loop_context.iteration(False):
+        pass
+      try:
+        dummy = self.model[path][COL_DUMMY]
+      except:
+        pass
+    if not dummy:
       self._selectExistingPath(path)
 
   def _onRowFilled(self, model, iter):
@@ -668,22 +880,28 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @type iter: gtk.TreeIter
     '''
     if iter and self._path_to_expand and \
-          self._path_to_expand[:-1] == model.get_path(iter):
+          self._path_to_expand[:-1] == model.get_path(iter).get_indices():
       self._selectExistingPath(self._path_to_expand)
       self._path_to_expand = None
 
-  def _selectExistingPath(self, path):
+  def _selectExistingPath(self, model_path):
     '''
     Select a path that already exists. Expand, scroll, and select.
     
-    @param path: Path to select.
+    @param model_path: Path to select, referring to the unfiltered model.
     @type path: tuple
     '''
-    if len(path) > 1:
-      self.expand_to_path(path[:-1])
-    self.scroll_to_cell(path)
+    iter = self.model.get_iter(model_path)
+    (res, filter_iter) = self.filter.convert_child_iter_to_iter(iter)
+    if not res:
+        return
+    filter_path = self.filter.get_path(filter_iter)
+    tree_path = gtk.TreePath(filter_path[:-1])
+    if len(filter_path) > 1:
+      self.expand_to_path(tree_path)
+    self.scroll_to_cell(filter_path)
     selection = self.get_selection()
-    selection.select_path(path)
+    selection.select_path(filter_path)
     
 
   def _onStartPop(self, model):
@@ -693,8 +911,9 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @param model: Model that emitted the signal.
     @type model: L{AccessibleModel}
     '''
-    if self.window:
-      self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+    if self.get_window():
+      window = self.get_window()
+      window.set_cursor(gdk.Cursor(gdk.CursorType.WATCH))
 
   def _onEndPop(self, model):
     '''
@@ -703,43 +922,9 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @param model: Model that emitted the signal.
     @type model: L{AccessibleModel}
     '''
-    if self.window:
-      self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.TOP_LEFT_ARROW))
-
-  def _accCellDataFunc(self, tvc, cellrenderer, model, iter):
-    '''
-    A cellrenderer data function. renderer's this application's node as insensitive.
-
-    @param tvc: A treeview column.
-    @type tvc: L{gtk.TreeViewColumn}
-    @param cellrenderer: The cellrenderer that needs to be tweaked.
-    @type cellrenderer: L{gtk.CellRenderer}
-    @param model: The treeview's data model.
-    @type model: L{AccessibleModel}
-    @param iter: The iter at the given row.
-    @type iter: L{gtk.TreeIter}
-    '''
-    if model.iter_is_valid(iter):
-      acc = model.get_value(iter, COL_ACC)
-    else:
-      acc = None
-    if self.isMyApp(acc):
-      cellrenderer.set_property('sensitive', False)
-    else:
-      cellrenderer.set_property('sensitive', True)
-
-  def _selectFunc(self, path):
-    '''
-    A selection function. Does not allow his application's node to be selected.
-
-    @param path: The path to the selected row.
-    @type path: tuple
-
-    @return: True if the row's accessible is not this app.
-    @rtype: boolean
-    '''
-    acc = self.model[path][COL_ACC]
-    return not self.isMyApp(acc)
+    if self.get_window():
+      window = self.get_window()
+      window.set_cursor(gdk.Cursor(gdk.CursorType.TOP_LEFT_ARROW))
 
   def _onRowActivated(self, treeview, path, view_column):
     '''
@@ -753,4 +938,5 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @param view_column: The column in the activated row.
     @type view_column: L{gtk.TreeViewColumn}
     '''
-    self.node.blinkRect()
+    if not self.isMyApp(self.filter[self.filter.get_iter(path)][COL_ACC]):
+      self.node.highlight()
