@@ -11,26 +11,28 @@ All rights reserved. This program and the accompanying materials are made
 available under the terms of the BSD which accompanies this distribution, and 
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import gtk
-import gtk.gdk
-import pyatspi
-import gobject
-import string
-import rsvg
+import gi
+
+from gi.repository import Gtk as gtk
+from gi.repository import Gdk as gdk
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository.Gio import Settings as GSettings
+gi.require_version('Rsvg', '2.0')
+from gi.repository import Rsvg as rsvg
 import cairo
-from tools import Tools, parseColorString
-import gconf
+import pyatspi
+import string
+from .tools import ToolsAccessor, parseColorString
 
-MAX_BLINKS = 6
-
-cl = gconf.client_get_default()
+gsettings = GSettings.new('org.a11y.Accerciser')
 BORDER_COLOR, BORDER_ALPHA = parseColorString(
-  cl.get_string('/apps/accerciser/highlight_border') or '#ff0000ff') 
+  gsettings.get_string('highlight-border'))
 
 FILL_COLOR, FILL_ALPHA  = parseColorString(
-  cl.get_string('/apps/accerciser/highlight_fill') or '#ff00006f')
+  gsettings.get_string('highlight-fill'))
 
-HL_DURATION = int(cl.get_float('/apps/accerciser/highlight_duration')*1000)
+HL_DURATION = int(gsettings.get_double('highlight-duration')*1000)
 
 class Bag(object):
   '''
@@ -40,9 +42,9 @@ class Bag(object):
     self.__dict__.update(kwargs)
     
   def __str__(self):
-    return ', '.join(vars(self).keys())
+    return ', '.join(list(vars(self).keys()))
 
-class Node(gobject.GObject, Tools):
+class Node(GObject.GObject, ToolsAccessor):
   '''
   Node class that contains convient references to accessibility information 
   for the currently selected node. A L{Node} instance will emit an 
@@ -57,19 +59,15 @@ class Node(gobject.GObject, Tools):
   @type extents: L{Bag}
   '''
   __gsignals__ = {'accessible-changed' : 
-                  (gobject.SIGNAL_RUN_FIRST,
-                   gobject.TYPE_NONE, 
-                   (gobject.TYPE_PYOBJECT,)),
-                  'blink-done' : 
-                  (gobject.SIGNAL_RUN_FIRST,
-                   gobject.TYPE_NONE, 
-                   ())}
+                  (GObject.SignalFlags.RUN_FIRST,
+                   None, 
+                   (GObject.TYPE_PYOBJECT,))}
   def __init__(self):
     self.desktop = pyatspi.Registry.getDesktop(0)
     self.acc = None
     self.extents = None
     self.tree_path = None
-    gobject.GObject.__init__(self)
+    GObject.GObject.__init__(self)
     
   def update(self, acc):
     '''
@@ -131,65 +129,22 @@ class Node(gobject.GObject, Tools):
                     2.0, 0)
     ah.highlight(HL_DURATION)
 
-  def blinkRect(self, times=MAX_BLINKS):
-    '''
-    Blink a rectangle on the screen using L{extents} for position and size.
-
-    @param times: Maximum times to blink.
-    @type times: integer
-    '''
-    if self.extents is None or \
-          -0x80000000 in (self.extents.x, self.extents.y):
-      return
-    self.max_blinks = times
-    self.blinks = 0
-    # get info for drawing higlight rectangles
-    display = gtk.gdk.display_get_default()
-    screen = display.get_default_screen()
-    self.root = screen.get_root_window()
-    self.gc = self.root.new_gc()
-    self.gc.set_subwindow(gtk.gdk.INCLUDE_INFERIORS)
-    self.gc.set_function(gtk.gdk.INVERT)
-    self.gc.set_line_attributes(3, gtk.gdk.LINE_ON_OFF_DASH, gtk.gdk.CAP_BUTT, 
-                                gtk.gdk.JOIN_MITER)
-    self.inv = gtk.Invisible()
-    self.inv.set_screen(screen)
-    gobject.timeout_add(30, self._drawRectangle)
-
-  def _drawRectangle(self):
-    '''
-    Draw a rectangle on the screen using L{extents} for position and size.
-    '''
-    # draw a blinking rectangle 
-    if self.blinks == 0:
-      self.inv.show()
-      self.inv.grab_add()
-    self.root.draw_rectangle(self.gc, False, 
-                             self.extents.x,
-                             self.extents.y,
-                             self.extents.width,
-                             self.extents.height)
-    self.blinks += 1
-    if self.blinks >= self.max_blinks:
-      self.inv.grab_remove()
-      self.inv.destroy()
-      self.emit('blink-done')
-      return False
-    return True
-
 class _HighLight(gtk.Window):
   '''
   Highlight box class. Uses compositing when available. When not, it does
   transparency client-side.
   '''
-  _svg = r"""
-<svg width="100%" height="100%" version="1.1"
-  xmlns="http://www.w3.org/2000/svg">
-  <rect x="$x" y="$y" width="$width" height="$height"
-      style="fill:$fill;stroke-width:$stroke_width;stroke:$stroke;
-      fill-opacity:$fill_opacity;stroke-opacity:$stroke_opacity"
-      rx="2" ry="2"
-      />
+  _svg = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
+<svg xmlns="http://www.w3.org/2000/svg">
+  <rect
+       style="fill:$fill;fill-opacity:$fill_opacity;fill-rule:evenodd;stroke:$stroke;stroke-width:2;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:$stroke_opacity"
+       id="highlight"
+       width="$width"
+       height="$height"
+       x="$x"
+       y="$y"
+       rx="2"
+       ry="2" />
 </svg>
 """
   def __init__(self, x, y, w, h, 
@@ -198,7 +153,7 @@ class _HighLight(gtk.Window):
                stroke_width, padding=0):
 
     # Initialize window.
-    gtk.Window.__init__(self, gtk.WINDOW_POPUP)
+    gtk.Window.__init__(self, type=gtk.WindowType.POPUP)
 
     # Normalize position for stroke and padding.
     self.x, self.y = x - padding, y - padding
@@ -209,12 +164,8 @@ class _HighLight(gtk.Window):
     if self._composited:
       # Prepare window for transparency.
       screen = self.get_screen()
-      colormap = screen.get_rgba_colormap()
-      self.set_colormap(colormap)
-    else:
-      # Take a screenshot for compositing on the client side.
-      self.root = gtk.gdk.get_default_root_window().get_image(
-        self.x, self.y, self.w, self.h)
+      visual = screen.get_rgba_visual()
+      self.set_visual(visual)
 
     # Place window, and resize it, and set proper properties.
     self.set_app_paintable(True)
@@ -230,47 +181,48 @@ class _HighLight(gtk.Window):
     self.svg = string.Template(self._svg).substitute(
       x=offset, y=offset,
       width=int(self.w - stroke_width), height=int(self.h - stroke_width),
-      fill=fill_color, 
+      fill=fill_color,
       stroke_width=stroke_width,
       stroke=stroke_color,
       fill_opacity=fill_alpha,
       stroke_opacity=stroke_alpha)
 
-    # Connect "expose" event.
-    self.connect("expose-event", self._onExpose)
+    # Connect "draw"
+    self.connect("draw", self._onExpose)
     
   def highlight(self, duration=500):
     if duration > 0:
-      gobject.timeout_add(duration, lambda w: w.destroy(), self)
+      GLib.timeout_add(duration, lambda w: w.destroy(), self)
       self.show_all()
     else:
       self.destroy()
     
-
   def _onExpose(self, widget, event):
     svgh = rsvg.Handle()
     try:
-      svgh.write(self.svg)
-    except (gobject.GError, KeyError, ValueError), ex:
-      print 'Error reading SVG for display: %s\r\n%s', ex, self.svg
+      svgh.write(bytes(self.svg, "utf-8"))
+    except (GObject.GError, KeyError, ValueError) as ex:
+      print ('Error reading SVG for display: %s\r\n%s', ex, self.svg)
       svgh.close()
       return
     svgh.close()
-      
+
     if not self._composited:
-      # Draw the screengrab of the underlaying window, and set the drawing
-      # operator to OVER.
-      self.window.draw_image(self.style.black_gc, self.root, 
-                             event.area.x,event.area.y, 
-                             event.area.x, event.area.y, 
-                             event.area.width, event.area.height)
       cairo_operator = cairo.OPERATOR_OVER
     else:
       cairo_operator = cairo.OPERATOR_SOURCE
-    cr = self.window.cairo_create()
+    window = self.get_window()
+    cr = window.cairo_create()
     cr.set_source_rgba(1.0, 1.0, 1.0, 0.0)
     cr.set_operator(cairo_operator)
     cr.paint()
 
-    svgh.render_cairo( cr )
+    svgh.render_cairo(cr)
     del svgh
+
+if __name__ == "__main__":
+    hl = _HighLight(200, 200, 200, 200, '#ff0000', 
+                    0.5, '#ff0000', 0.996108949416, 8.0, 0)
+    hl.highlight(2000)
+    gtk.main()
+
